/* solution to problem C01 from the "many digits friendly competition":
   compute the first 10^N decimal digits after the decimal point of
   sin(tan(cos(1))). */

#include "many.h"

int
main (int argc, char *argv[])
{
  unsigned long N, M;
  mp_prec_t p;
  mpfr_t i, j;
  char *lo;
  mp_exp_t exp_lo;
  int st, st0, verbose = 0;

  if (argc >= 2 && strcmp (argv[1], "-v") == 0)
    {
      verbose = 1;
      argv ++;
      argc --;
    }

  N = atoi (argv[1]);

  fprintf (stderr, "Using GMP %s and MPFR %s\n", gmp_version, mpfr_version);
  st = cputime ();

  mpfr_init (i);
  mpfr_init (j);

  M = N;

  do
    {
      M += 10;
      mpfr_set_prec (j, 32);
      mpfr_set_d (j, LOG2_10, GMP_RNDU);
      mpfr_mul_ui (j, j, M, GMP_RNDU);
      mpfr_add_ui (j, j, 2, GMP_RNDU);
      p = mpfr_get_ui (j, GMP_RNDU);
      if (verbose)
        fprintf (stderr, "Setting precision to %lu\n", p);

      mpfr_set_prec (i, p);
      mpfr_set_prec (j, 2);
      mpfr_set_ui (j, 1, GMP_RNDN);
      st0 = cputime ();
      mpfr_cos (i, j, GMP_RNDN);
      if (verbose)
        fprintf (stderr, "cos() took %dms\n", cputime () - st0);
      st0 = cputime ();
      mpfr_tan (i, i, GMP_RNDN);
      if (verbose)
        fprintf (stderr, "tan() took %dms\n", cputime () - st0);
      st0 = cputime ();
      mpfr_sin (i, i, GMP_RNDN);
      if (verbose)
        fprintf (stderr, "sin() took %dms\n", cputime () - st0);
      st0 = cputime ();
      lo = mpfr_get_str (NULL, &exp_lo, 10, M, i, GMP_RNDN);
      if (verbose)
        fprintf (stderr, "get_str() took %dms\n", cputime () - st0);
    }
  while (can_round (lo, N, M) == 0);

  lo[N] = '\0';

  printf ("%s\n", lo);

  mpfr_clear (i);
  mpfr_clear (j);

  fprintf (stderr, "Cputime: %dms\n", cputime () - st);
  return 0;
}
