#include "many.h"

/* Factor > 1 between real value and computed value: < (1 + 2(^-p))^r */

static void
compval (mpfr_t x, mp_prec_t p)
{
  mpfr_t y;

  mpfr_set_prec (x, p);
  mpfr_init2 (y, p);
  mpfr_const_pi (x, GMP_RNDN); /* r = 1 */

  mpfr_sqr (y, x, GMP_RNDN); /* r = 3 */
  mpfr_sqr (y, y, GMP_RNDN); /* r = 7 */
  mpfr_mul (x, x, y, GMP_RNDN); /* r = 9 */

  mpfr_sqr (y, x, GMP_RNDN); /* r = 19 */
  mpfr_sqr (y, y, GMP_RNDN); /* r = 39 */
  mpfr_mul (x, x, y, GMP_RNDN); /* r = 49 */

  mpfr_sqr (y, x, GMP_RNDN); /* r = 99 */
  mpfr_sqr (y, y, GMP_RNDN); /* r = 199 */
  mpfr_mul (x, x, y, GMP_RNDN); /* r = 249 */

  mpfr_sqr (x, x, GMP_RNDN); /* r = 499 */
  mpfr_sqr (x, x, GMP_RNDN); /* r = 999 */
  mpfr_sqr (x, x, GMP_RNDN); /* r = 1999 -> 11 guard bits */

  mpfr_clear (y);
}

int
main (int argc, char *argv[])
{
  unsigned long N = atoi (argv[1]), M;
  mp_prec_t p;
  mpfr_t i, j;
  char *lo;
  mp_exp_t exp_lo;
  int st, st0;

  fprintf (stderr, "Using GMP %s and MPFR %s\n", gmp_version, mpfr_version);
  st = cputime ();

  mpfr_init (i);
  mpfr_init (j);

  M = N;

  do
    {
      M += 10;
      mpfr_set_prec (i, 32);
      mpfr_set_d (i, LOG2_10, GMP_RNDU);
      mpfr_mul_ui (i, i, M, GMP_RNDU);
      mpfr_add_ui (i, i, 1652 + 11, GMP_RNDU);
      p = mpfr_get_ui (i, GMP_RNDU);
      fprintf (stderr, "Setting precision to %lu\n", p);
      compval (i, p);
      st0 = cputime ();
      lo = mpfr_get_str (NULL, &exp_lo, 10, M + 498, i, GMP_RNDN);
      st0 = cputime () - st0;
    }
  while (can_round (lo + 498, N, M) == 0);

  lo += 498;
  lo[N] = '\0';
  printf ("%s\n", lo);

  mpfr_clear (i);

  fprintf (stderr, "Cputime: %dms (output %dms)\n", cputime () - st, st0);
  return 0;
}
