#include "many.h"

/*  C08   -   Sin(6^6^6)

    c08a  -   Jim White -  base version adapted direct from p08a 
                           magnitude is too huge for mpfr_sin to deal
                           with, and trisections/bisections are simply
                           too numerous, so we scale the value down 
                           to  be < 2Pi and pass that remainder on 
                           to mpfr_sin
 */

int
main (int argc, char *argv[])
{
  unsigned long N = atoi (argv[1]), M;
  mp_prec_t p;
  mpfr_t i, j, k, m, pi;
  char *lo;
  mp_exp_t exp_lo;
  int st, st0;

  fprintf (stderr, "Using GMP %s and MPFR %s\n", gmp_version, mpfr_version);
  st = cputime ();

  mpfr_init (i);  /* target result precision */
  mpfr_init (pi); /* pi to target precision */
  mpfr_init (j);  /* argument 6^(6^6), integer requiring 120640 bits (3769) */
  mpfr_init (k);  /* to get accurate remainder of j / 2PI we need a register of width prec(i) + prec(j) ? */
  mpfr_init (m);

  M = N;

  do
    {
      M += 10;
      mpfr_set_prec (i, 32);
      mpfr_set_d (i, LOG2_10, GMP_RNDU);
      mpfr_mul_ui (i, i, M, GMP_RNDU);
      p = mpfr_get_ui (i, GMP_RNDU);
      fprintf (stderr, "Setting precision to %lu\n", p);

      mpfr_set_prec (i, p);
      mpfr_set_prec (j, 120640);    /*  6^6^6  fits in this  */
      mpfr_set_prec (m, 120640);
      mpfr_set_prec (k,  p + 120640);
      mpfr_set_prec (pi, p + 120640);

      mpfr_const_pi (pi, GMP_RNDN);
      mpfr_mul_ui (pi, pi, 2, GMP_RNDN);         /*  pi = 2PI      */
      mpfr_ui_pow_ui (j, 6, 46656, GMP_RNDN);    /*  j  = 6^(6^6)  */

      mpfr_div   (k, j, pi, GMP_RNDN);           /*  k  = j / pi   */
      mpfr_trunc (m, k);                         /*  m  = floor(k) */
      mpfr_mul (k, pi, m, GMP_RNDN);             /*  k  = m * pi   */
      mpfr_sub (k, j, k, GMP_RNDN);              /*  k  = j - k    */
      mpfr_prec_round (k, p, GMP_RNDN);          /*  reduce it to target precision */

      mpfr_sin (i, k, GMP_RNDN);

      st0 = cputime ();
      lo = mpfr_get_str (NULL, &exp_lo, 10, M, i, GMP_RNDN);
      st0 = cputime () - st0;
    }
  while (can_round (lo + 1, N, M) == 0);

  /* lo ++;  this result is positive */
  lo[N] = '\0';
  printf ("%s\n", lo);

  mpfr_clear (i);
  mpfr_clear (j);

  fprintf (stderr, "Cputime: %dms (output %dms)\n", cputime () - st, st0);
  return 0;
}
