/* solution to problem C10 from the "many digits friendly competition":
   compute the first 10^N decimal digits after the decimal point of
   (7+2^(1/5)-5*8^(1/5))^(1/3) + 4^(1/5) - 2^(1/5).

   With x = 2^(1/5), this expression is
   (7 + x - 5*x^3)^(1/3) + x^2 - x.

   We compute with precision p and rounding to nearest:

   x = o(2^(1/5))
   y = o(x^2)
   z = o(y*x)
   q = o(7+x)
   r = o(5*z)
   s = o(q-r)
   t = o(s^(1/3))
   u = o(t+y)
   v = o(u-x).

   p=10;
   x = [approx(2^(1/5),p,3),approx(2^(1/5),p,2)]
   y = [approx(x[1]^2,p,3),approx(x[2]^2,p,2)]
   z = [approx(y[1]*x[1],p,3),approx(y[2]*x[2],p,2)]
   q = [approx(7+x[1],p,3),approx(7+x[2],p,2)]
   r = [approx(5*z[1],p,3),approx(5*z[2],p,2)]
   s = [approx(q[1]-r[2],p,3),approx(q[2]-r[1],p,2)]
   t = [approx(s[1]^(1/3),p,3),approx(s[2]^(1/3),p,2)]
   u = [approx(t[1]+y[1],p,3),approx(t[2]+y[2],p,2)]
   v = [approx(u[1]-x[2],p,3),approx(u[2]-x[1],p,2)]

   For p >= 10, we have 1 <= x, y, z < 2, 8 <= q < 16,
   4 <= r < 8, 1/2 <= s, t < 1, 2 <= u < 4, 1/2 <= v <= 2.
   More precisely, 67/128 <= s <= 19/32.

   We have x = 2^(1/5) * (1+theta), and the absolute error on x can be
   bounded by 2^(1/5) * 2^(-p) <= 2^(1-p).

           y = 2^(2/5) * (1+theta)^3
           err(y) <= 2^(2/5) * (4*theta) <= 6 * 2^(-p).
	   z = 2^(3/5) * (1+theta)^5

   err(q) <= 1/2*ulp(q) + err(x) <= 2^(3-p) + 2^(1-p) <= 10 * 2^(-p).

   r = 5*2^(3/5) * (1+theta)^6
   Since (1+theta)^6 can be written 1+7*theta, the absolute error on r
   is bounded by 5*2^(3/5)*7*theta <= 54 * 2^(-p).

   err(s) <= 1/2*ulp(s) + err(q) + err(r) <= 
   2^(-p-1) + 10 * 2^(-p) + 54 * 2^(-p) <= 65 * 2^(-p).

   err(t) <= 1/2*ulp(t) + err(s) * (1/3*alpha^(-2/3))
          <= 2^(-p-1) + 65 * 2^(-p) * (1/3*(67/128)^(-2/3))
	  <= 34 * 2^(-p).
   
   err(u) <= 1/2*ulp(u) + err(t) + err(y)
          <= 2^(1-p) + 34 * 2^(-p) + 6 * 2^(-p) <= 42 * 2^(-p).

   err(v) <= 1/2*ulp(v) + err(u) + err(x)
          <= 2^(-p) + 42 * 2^(-p) + 2^(1-p)
	  <= 45 * 2^(-p) <= 2^(6-p).
*/

#define WANT_MPFR_ROOT
#include "many.h"

int
main (int argc, char *argv[])
{
  unsigned long N = atoi (argv[1]);
  mp_prec_t p;
  mpfr_t i, j, k, l;
  char *lo;
  int st, st0;

  fprintf (stderr, "Using GMP %s and MPFR %s\n", gmp_version, mpfr_version);
  st = cputime ();

  mpfr_init2 (i, 32);

  mpfr_set_d (i, LOG2_10, GMP_RNDU);
  mpfr_mul_ui (i, i, N, GMP_RNDU);
  mpfr_add_ui (i, i, 7, GMP_RNDU);
  p = mpfr_get_ui (i, GMP_RNDU);
  if (p < 10)
    p = 10;
  fprintf (stderr, "Setting precision to %lu\n", p);

  mpfr_set_prec (i, p);
  mpfr_init2 (j, p);
  mpfr_init2 (k, p);
  mpfr_init2 (l, p);

  mpfr_set_ui (i, 2, GMP_RNDN);
  mpfr_root (i, i, 5, GMP_RNDN); /* x = 2^(1/5) */
  mpfr_mul (j, i, i, GMP_RNDN); /* y */
  mpfr_mul (k, j, i, GMP_RNDN); /* z */
  mpfr_add_ui (l, i, 7, GMP_RNDN); /* q */
  mpfr_mul_ui (k, k, 5, GMP_RNDN); /* r */
  mpfr_sub (l, l, k, GMP_RNDN); /* s */
  mpfr_root (l, l, 3, GMP_RNDN); /* t */
  mpfr_add (l, l, j, GMP_RNDN); /* u */
  mpfr_sub (i, l, i, GMP_RNDN); /* v */

  st0 = cputime ();
  lo = fixed_point_output (i, N);
  st0 = cputime () - st0;

  printf ("%s\n", lo);
  free (lo);

  mpfr_clear (i);
  mpfr_clear (j);
  mpfr_clear (k);
  mpfr_clear (l);

  fprintf (stderr, "Cputime: %dms (output %dms)\n", cputime () - st, st0);
  return 0;
}
