#include "many.h"

static void
compval (mpfr_t x, mp_prec_t p)
{
  mpfr_t y, i;

  mpfr_set_prec (x, p);
  mpfr_init2 (y, p);
  mpfr_init2 (i, 2);

  mpfr_set_ui (i, 2, GMP_RNDN);
  mpfr_sqrt (x, i, GMP_RNDN); /* err < 2^(-p) */
  mpfr_tan (x, x, GMP_RNDN); /* err < (42 + 4) * 2^(-p) = 46 * 2^(-p) */
  mpfr_set_ui (i, 1, GMP_RNDN);
  mpfr_sin (y, i, GMP_RNDN); /* err < 1/2 * 2^(-p) */
  mpfr_atanh (y, y, GMP_RNDN); /* err < (2 + 1) * 2^(-p) = 3 * 2^(-p) */
  mpfr_add (x, x, y, GMP_RNDN); /* err < 53 * 2^(-p) -> 6 guard bits */

  mpfr_clear (y);
  mpfr_clear (i);
}

int
main (int argc, char *argv[])
{
  unsigned long N = atoi (argv[1]), M;
  mp_prec_t p;
  mpfr_t i, j;
  char *lo;
  mp_exp_t exp_lo;
  int st, st0;

  fprintf (stderr, "Using GMP %s and MPFR %s\n", gmp_version, mpfr_version);
  st = cputime ();

  mpfr_init (i);
  mpfr_init (j);

  M = N;

  do
    {
      M += 10;
      mpfr_set_prec (i, 32);
      mpfr_set_d (i, LOG2_10, GMP_RNDU);
      mpfr_mul_ui (i, i, M, GMP_RNDU);
      mpfr_add_ui (i, i, 6, GMP_RNDU);
      p = mpfr_get_ui (i, GMP_RNDU);
      fprintf (stderr, "Setting precision to %lu\n", p);
      compval (i, p);
      st0 = cputime ();
      lo = mpfr_get_str (NULL, &exp_lo, 10, M + 1, i, GMP_RNDN);
      st0 = cputime () - st0;
    }
  while (can_round (lo + 1, N, M) == 0);

  lo++;
  lo[N] = '\0';
  printf ("%s\n", lo);

  mpfr_clear (i);

  fprintf (stderr, "Cputime: %dms (output %dms)\n", cputime () - st, st0);
  return 0;
}
