#include "many.h"

#define LOG2_39P99 5.3215674 /* upper approximation to log(39.99)/log(2) */
#define LOG2_39P99L 1.9996392 /* lower approximation of log2(3.999) */

/* return ceil(log(n)/log(2)) */
unsigned long
uceil_log2 (unsigned long n)
{
  unsigned long l;

  l = 0;
  while (n > 1)
    {
      l ++;
      n = (n + 1) / 2;
    }
  return l;
}

int
main (int argc, char *argv[])
{
  unsigned long N = atoi (argv[1]), M, log2M;
  mp_prec_t p, pn;
  mpfr_t i, j;
  char *lo;
  mp_exp_t exp_lo;
  int k;
  int st, st0;

  fprintf (stderr, "Using GMP %s and MPFR %s\n", gmp_version, mpfr_version);
  st = cputime ();

  mpfr_init (i);
  mpfr_init (j);

  M = N;

  do
    {
      M += 10;
      mpfr_set_prec (i, 32);
      mpfr_set_prec (j, 32);
      mpfr_set_d (i, LOG2_39P99, GMP_RNDU);
      mpfr_mul_ui (i, i, M, GMP_RNDU);
      mpfr_add_ui (i, i, uceil_log2 (M) + 1, GMP_RNDU);
      p = mpfr_get_ui (i, GMP_RNDU);
      fprintf (stderr, "Setting precision to %lu\n", p);

      mpfr_set_prec (i, p);
      mpfr_set_prec (j, p);

      mpfr_set_d (i, 0.5, GMP_RNDN);
      for (k = 0; k < (int) N; k++)
        {
          mpfr_sqr (j, i, GMP_RNDN);
          mpfr_sub (i, i, j, GMP_RNDN);
          mpfr_mul_ui (i, i, 3999, GMP_RNDN);
          mpfr_div_ui (i, i, 1000, GMP_RNDN);
          /* reduce precision */
          if (k % 4 == 0)
            {
              pn = p - (unsigned long) ((double) k * LOG2_39P99L);
              mpfr_prec_round (i, pn, GMP_RNDN);
              mpfr_prec_round (j, pn, GMP_RNDN);
            }
        }

      st0 = cputime ();
      lo = mpfr_get_str (NULL, &exp_lo, 10, M, i, GMP_RNDN);
      st0 = cputime () - st0;
    }
  while (can_round (lo, N, M) == 0);

  fprintf (stderr, "pn=%u\n", pn);

  lo[N] = '\0';
  printf ("%s\n", lo);

  mpfr_clear (i);
  mpfr_clear (j);

  fprintf (stderr, "Cputime: %dms (output %dms)\n", cputime () - st, st0);
  return 0;
}
