#include "many.h"

int
main (int argc, char *argv[])
{
  unsigned long N = atoi (argv[1]), M, l;
  mp_prec_t p;
  mpfr_t i, j;
  char *lo;
  mp_exp_t e, f;
  unsigned long k;
  int st, st0;

  fprintf (stderr, "Using GMP %s and MPFR %s\n", gmp_version, mpfr_version);
  st = cputime ();

  mpfr_init (i);
  mpfr_init (j);

  M = N;

  do
    {
      M += 10;
      mpfr_set_prec (i, 32);
      mpfr_set_d (i, 9.33, GMP_RNDU); /* upper approximation of
                                         log(640)/log(2) */
      mpfr_mul_ui (i, i, M, GMP_RNDU);
      p = mpfr_get_ui (i, GMP_RNDU);
      if (argc > 2)
        p += atoi (argv[2]);
      fprintf (stderr, "Setting precision to %lu\n", p);

      mpfr_set_prec (i, p);
      mpfr_set_prec (j, p);

      mpfr_set_ui (j, 1, GMP_RNDN);
      for (k = N; k >= 1; k--)
	{
          mpfr_mul_ui (j, j, k, GMP_RNDN);
          mpfr_mul_ui (j, j, k, GMP_RNDN);
          mpfr_mul_ui (j, j, k, GMP_RNDN);
          mpfr_div_ui (j, j, 2 * k + 1, GMP_RNDN);
          mpfr_div_ui (j, j, 2 * k + 1, GMP_RNDN);
          mpfr_div_ui (j, j, 2 * k + 1, GMP_RNDN);
	  mpfr_add_ui (j, j, 21 * k - 8, GMP_RNDN);
          mpfr_div_2ui (j, j, 3, GMP_RNDN);
	}
      mpfr_const_pi (i, GMP_RNDN);
      mpfr_sqr (i, i, GMP_RNDN);
      mpfr_div_ui (i, i, 6, GMP_RNDN);
      mpfr_sub (i, i, j, GMP_RNDN);

      for (f = 0, l = 7 * N + 1; l > 1; f ++, l = (l + 9) / 10);
      f += (mp_exp_t) ceil ((3.0 - (double) p) * LOG10_2);
      /* the absolute error on i is bounded by (7N+1)*2^(2-p) <= 1/2*10^f */

      st0 = cputime ();
      lo = mpfr_get_str (NULL, &e, 10, M, i, GMP_RNDN);
      st0 = cputime () - st0;

      /* if f <= e - M, then the error on i is less than 1/2*ulp(lo) */
      if (f > e)
	abort ();
      if (f > e - (mp_exp_t) M)
	M = e - f;
    }
  while (can_round (lo, N, M) == 0);

  lo[N] = '\0';
  printf ("%s\n", lo);

  mpfr_clear (i);
  mpfr_clear (j);

  fprintf (stderr, "Cputime: %dms (output %dms)\n", cputime () - st, st0);
  return 0;
}
