#define USE_MPFI
#include "many.h"

int
main (int argc, char *argv[])
{
  unsigned long N = atoi (argv[1]);
  mp_prec_t p;
  mpfi_t i, j;
  mpfr_t k;
  char *lo, *hi;
  mp_exp_t exp_lo, exp_hi;
  int st;

  fprintf (stderr, "Using GMP %s and MPFR %s\n", gmp_version, mpfr_version);
  st = cputime ();

  /* initial estimate of the working precision */
  p = (mp_prec_t) (3.321928095 * (double) N + 10.0);
  fprintf (stderr, "Setting initial precision to %lu\n", p);

  mpfi_init (i);
  mpfi_init (j);
  mpfr_init2 (k, 3);

  do
    {
      mpfi_set_prec (i, p);
      mpfi_set_prec (j, p);

      /* i = 2 * zeta(2) - 1 */
      mpfr_set_ui (k, 2, GMP_RNDN);
      mpfr_zeta (LEFT(i), k, GMP_RNDD);
      mpfr_mul_2ui (LEFT(i), LEFT(i), 1, GMP_RNDN); /* no mantissa change */
      mpfr_sub_ui (LEFT(i), LEFT(i), 1, GMP_RNDN); /* no exponent change */
      mpfr_set (RIGHT(i), LEFT(i), GMP_RNDU);
      mpfr_nextabove (RIGHT(i));

      /* j = zeta(3) - 1 */
      mpfr_set_ui (k, 3, GMP_RNDN);
      mpfr_zeta (LEFT(j), k, GMP_RNDD);
      mpfr_set (RIGHT(j), LEFT(j), GMP_RNDU);
      mpfr_nextabove (RIGHT(j));
      mpfi_sub_ui (j, j, 1);

      mpfi_mul (i, i, j);

      /* j = zeta(5) - 1 */
      mpfr_set_ui (k, 5, GMP_RNDN);
      mpfr_zeta (LEFT(j), k, GMP_RNDD);
      mpfr_set (RIGHT(j), LEFT(j), GMP_RNDU);
      mpfr_nextabove (RIGHT(j));
      mpfi_sub_ui (j, j, 1);

      mpfi_add (i, i, j);
      mpfi_add (i, i, i);

      lo = mpfr_get_str (NULL, &exp_lo, 10, N, LEFT(i), GMP_RNDZ);
      hi = mpfr_get_str (NULL, &exp_hi, 10, N, RIGHT(i), GMP_RNDZ);
      if (exp_lo == exp_hi && strcmp (lo, hi) == 0)
        break;
      else
        {
          p += 10;
          fprintf (stderr, "Increasing precision to %lu\n", p);
        }
    }
  while (1);

  printf ("%s\n", lo);

  mpfi_clear (i);
  mpfi_clear (j);
  mpfr_clear (k);

  fprintf (stderr, "Cputime: %dms\n", cputime () - st);
  return 0;
}
