/* solution to problem C19 from the "many digits friendly competition":
   compute the first 10^N decimal digits after the decimal point of
   L = sum((1/7)^(n^3+1), n=1..infinity). */

#include "many.h"

int
main (int argc, char *argv[])
{
  unsigned long N = atoi (argv[1]), M;
  mp_prec_t p, q, j;
  mpfr_t i;
  char *lo;
  mp_exp_t exp_lo;
  int st, st0;

  fprintf (stderr, "Using GMP %s and MPFR %s\n", gmp_version, mpfr_version);
  st = cputime ();

  mpfr_init (i);

  M = N;

  do
    {
      M += 10;
      mpfr_set_prec (i, 32);

      /* first compute q = ceil(1 + M*log(10)/log(7)) */
      mpfr_set_d (i, LOG7_10, GMP_RNDU);
      mpfr_mul_ui (i, i, M, GMP_RNDU);
      mpfr_add_ui (i, i, 1, GMP_RNDU);
      q = mpfr_get_ui (i, GMP_RNDU);
      fprintf (stderr, "Truncating L to %lu digits\n", q);

      lo = (char*) malloc (q + 3);
      memset (lo, '0', q + 2);
      lo[1] = '.';
      lo[q + 2] = '\0';

      for (j = 1; j * j * j < q; j++)
	lo[j * j * j + 2] = '1';

      mpfr_set_d (i, LOG2_10, GMP_RNDU);
      mpfr_mul_ui (i, i, M, GMP_RNDU);
      mpfr_add_ui (i, i, 1, GMP_RNDU);
      p = mpfr_get_ui (i, GMP_RNDU);
      fprintf (stderr, "Setting precision to %lu\n", p);

      mpfr_set_prec (i, p);

      mpfr_set_str (i, lo, 7, GMP_RNDN);

      free (lo);

      st0 = cputime ();
      lo = mpfr_get_str (NULL, &exp_lo, 10, M, i, GMP_RNDN);
      st0 = cputime () - st0;
    }
  while (can_round (lo, N - 1, M) == 0);

  lo[N-1] = '\0';
  printf ("0%s\n", lo);

  mpfr_clear (i);

  fprintf (stderr, "Cputime: %dms (output %dms)\n", cputime () - st, st0);
  return 0;
}

