#include "many.h"

int
main (int argc, char *argv[])
{
  unsigned long N = atoi (argv[1]), M;
  int st, st0;
  mpfr_t x, y, z, u;
  mp_prec_t Q, p, q, newq;
  char *lo;
  mp_exp_t exp_lo;

  fprintf (stderr, "Using GMP %s and MPFR %s\n", gmp_version, mpfr_version);
  st = cputime ();

  mpfr_init2 (x, 32);
  mpfr_init (y);
  mpfr_init (z);
  mpfr_init (u);

  M = N + 10;

  mpfr_set_d (x, LOG2_10, GMP_RNDU);
  mpfr_mul_ui (x, x, M, GMP_RNDU);
  Q = mpfr_get_ui (x, GMP_RNDU);
  fprintf (stderr, "Target precision is %lu\n", Q);

  /* initialization */
  mpfr_set_prec (x, 2);
  mpfr_set_d (x, 1.3, GMP_RNDN);
  q = 2;

  /* invariant: we have |x-rho| <= 2^(-q), it is true here since rho ~ 1.302 */
  while (1)
    {
      if (q < Q)
	{
	  for (newq = Q; newq > 2 * q; newq = (newq + 1) / 2);
	  /* now newq <= 2 * q */
	}
      else
	{
	  M += 10;
	  mpfr_set_d (x, LOG2_10, GMP_RNDU);
	  mpfr_mul_ui (x, x, M, GMP_RNDU);
	  Q = mpfr_get_ui (x, GMP_RNDU);
	  fprintf (stderr, "Target precision is %lu\n", Q);
	  newq = Q;
	}

      p = newq + 4;

      /* start iteration */
      mpfr_prec_round (x, p, GMP_RNDN); /* extend x to p bits */
      mpfr_set_prec (y, p);
      mpfr_set_prec (z, p);
      mpfr_set_prec (u, p);
      mpfr_sin_cos (y, z, x, GMP_RNDN);
      mpfr_exp (z, z, GMP_RNDN); /* t */
      mpfr_sub (u, x, z, GMP_RNDN);
      mpfr_mul (y, y, z, GMP_RNDN); /* v */
      mpfr_add_ui (y, y, 1, GMP_RNDN); /* w */
      mpfr_div (y, u, y, GMP_RNDN);
      mpfr_sub (x, x, y, GMP_RNDN);

      /* try is precision is enough */
      q = newq;

      if (q >= Q)
	{
	  st0 = cputime ();
	  lo = mpfr_get_str (NULL, &exp_lo, 10, M + 1, x, GMP_RNDN);
	  st0 = cputime () - st0;
	  if (can_round (lo + 1, N, M))
	    break;
	}
    }

  lo ++; /* to skip the initial 1 */
  lo[N] = '\0';
  printf ("%s\n", lo);

  mpfr_clear (x);
  mpfr_clear (y);
  mpfr_clear (z);
  mpfr_clear (u);

  fprintf (stderr, "Cputime: %dms (output %dms)\n", cputime () - st, st0);
  return 0;
}
