#include "many.h"
#include "crq.h"

/* On [0, 1], |f^{(100 n)}| <= 2^{BD[n]}
 * THIS IS STILL A CONJECTURE */
unsigned long BD[] = {
    2, 451, 1071, 1760, 2494, 3261, 4055, 4871 };

/* Bound the grows of f^{2n} between 100k and 100k + 100 */
unsigned long growth[] = { 5, 7, 7, 8, 8, 8, 9, 9 };

unsigned long bound_f2n (unsigned long n)
{
    unsigned long step, rem;
    n = n * 2;
    if ((n > 800) || (n < 2))
	abort ();
	
    step = n / 100;
    rem = n % 100;
    return BD[step] + rem * growth[step];
}

void sin3 (mpfr_t res, mpfr_t x, mp_prec_t rndm)
{
    mpfr_set_prec (res, mpfr_get_prec (res) + 10);
    mpfr_sin (res, x, rndm);
    mpfr_sin (res, res, rndm);
    mpfr_sin (res, res, rndm);
    mpfr_prec_round (res, mpfr_get_prec (res) - 10, GMP_RNDN);
}

int main (int argc, char *argv[])
{
    int total_time;
    mpfr_t a, b, res, reslo, reshi, M, un;
    mpfr_t matherror, evalerror, totalerror;
    mp_prec_t p;
    char *lo, *hi;
    mp_exp_t exp_lo, exp_hi;
    char *filename;
    unsigned long m, n, wp, correct_bits;
    unsigned long N;
    struct crq_stats stats;

    fprintf (stderr, "Using GMP %s and MPFR %s\n", gmp_version, mpfr_version);
    total_time = cputime ();
    stats.pol_time = 0;
    stats.usp_time = 0;
    stats.refine_time = 0;
    stats.coeff_time = 0;
    stats.newton_good = 0;
    stats.newton_bad = 0;
    stats.dich_prep_time = 0;
    stats.dicho_time = 0;
    stats.newton_failures = 0;

    mpfr_init (a);
    mpfr_init (b);
    mpfr_init (res);
    mpfr_init (reshi);
    mpfr_init (reslo);
    mpfr_init (evalerror);
    mpfr_init (matherror);
    mpfr_init (totalerror);
    mpfr_init (M);
    mpfr_init (un);
    mpfr_set_ui (a, 0, GMP_RNDN);
    mpfr_set_ui (b, 1, GMP_RNDN);
    mpfr_set_ui (un, 1, GMP_RNDN);
    /* m, n, wp "opt" m = 133; n = 150; wp = 3490; */
    if (argc > 3) {
	m = atoi (argv[3]);
	n = atoi (argv[2]);
	N = atoi (argv[1]);
    } else if (argc > 2) {
	m = 8;
	n = atoi (argv[2]);
	N = atoi (argv[1]);
    } else if (argc > 1) {
	m = 8;
	n = 278;
	N = atoi (argv[1]);
    } else {
	m = 8;
	n = 278;
	N = 1000;
    }

    p = (mp_prec_t) (LOG2_10 * (double) N + 10.0);
    wp = p + 42;
    /* p is the expected binary accuracy to reach to be able to get
     * an accuracy of N decimal digits. wp is the binary working precision */
    do {
#ifdef DEBUG
	fprintf (stderr, "Starting GL with n = %ld, m = %ld, wp = %ld\n", n,
		 m, wp);
#endif
	mpfr_set_prec (res, wp);
	/* Do we want to reuse a precomputed table ? */
	if (argc >= 5) {
	    filename = argv[4];
	    fprintf (stderr, "Creating/reusing precomputed table %s\n",
		     argv[4]);
	} else
	    filename = NULL;
	compose_gl_stats (res, evalerror, a, b, m, n, sin3, un, wp, 0,
			  filename, &stats);
	mpfr_set_ui (M, 1, GMP_RNDN);
	mpfr_mul_2si (M, M, bound_f2n (n), GMP_RNDU);
	gl_error (matherror, a, b, m, n, M);
	mpfr_add (totalerror, matherror, evalerror, GMP_RNDU);
	correct_bits = mpfr_get_exp (res) - mpfr_get_exp (totalerror);
#ifdef DEBUG
	fprintf (stderr, "Correct bits = %ld, wanted %ld\n", correct_bits, p);
	fprintf (stderr, "Exp res = %ld", mpfr_get_exp (res));
	fprintf (stderr, " exp error = %ld\n", mpfr_get_exp (totalerror));
	mpfr_dump (totalerror);
#endif
	if (correct_bits >= p) {
	    mpfr_set_prec (reslo, wp);
	    mpfr_set_prec (reshi, wp);
	    mpfr_sub (reslo, res, totalerror, GMP_RNDD);
	    mpfr_add (reshi, res, totalerror, GMP_RNDU);
	    lo = mpfr_get_str (NULL, &exp_lo, 10, N, reslo, GMP_RNDZ);
	    hi = mpfr_get_str (NULL, &exp_hi, 10, N, reshi, GMP_RNDZ);
	    if ((exp_lo == exp_hi) && (strcmp (lo, hi) == 0))
		break;
	    else { /* p insufficient */
		p += p / 10;
		wp += wp / 10;
	    }
	} else {
	    if (mpfr_cmp (evalerror, matherror) > 0)
		wp += wp / 10;
	    else
		n += n / 10;
	}
    } while (42 == 42);

    printf ("%s\n", lo);
    total_time = cputime () - total_time;
    fprintf (stderr, "Cputime: %dms\n", total_time);
#ifdef WANT_STATS
    printf ("%d [Polynomial computation time]\n", stats.pol_time);
    printf ("%d [Uspensky time]\n", stats.usp_time);
    printf ("%d [Root refinement time]\n", stats.refine_time);
    printf ("%d [Coefficient computation time]\n", stats.coeff_time);
    printf ("%d [Useful newton time]\n", stats.newton_good);
    printf ("%d [Useless newton time]\n", stats.newton_bad);
    printf ("%d [dicho preparation time]\n", stats.dich_prep_time);
    printf ("%d [dicho time]\n", stats.dicho_time);
    printf ("%d [newton failures]\n", stats.newton_failures);
#endif
    return 0;
}
