/*
Date: Tue, 4 Oct 2005 08:57:51 +0200
From: Guillaume Hanrot <Guillaume.Hanrot@loria.fr>
To: Paul Zimmermann <Paul.Zimmermann@loria.fr>
Subject: pour la gloire

...la solution logarithmique. Pour completer, il faut prendre N en
mpz_t, et gerer la precision en fonction de N. Quelque chose du
genre 2*mpz_sizeinbase(N, 2) + 50 doit suffire.

Guillaume
*/

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#define USE_MPFI
#include "many.h"

#define THRES_EXP (1 << 25)
#define THRESHOLD 100

int
main (int argc, char *argv[])
{
  unsigned long k, ok, flag = 0;
  mp_prec_t p;
  char *lo, *hi;
  mp_exp_t exp_lo, exp_hi;
  int st;

  fprintf (stderr, "Using GMP %s and MPFR %s\n", gmp_version, mpfr_version);
  st = cputime ();


  if (strlen(argv[1]) < 3)
    {
      unsigned long N = strtoul (argv[1], NULL, 10);

      /* initial estimate of the working precision */
      p = (argc > 2 ? strtoul (argv[2], NULL, 10) :
           (N <= 10000000 ? 100 : 10000));
      fprintf (stderr, "Setting initial precision to %lu\n", p);

      if (errno)
        {
          perror ("c23-fin");
          exit (1);
        }

      if (N < THRESHOLD) {
        mpfi_t fib0, fib1, fibtmp;
        mpfi_t num, den, renorm;

        mpfi_init2(fib0, p);
        mpfi_init2(fib1, p);
        mpfi_init2(fibtmp, p);
        mpfi_init2(den, p);
        mpfi_init2(num, p);

        do
          {
            mpfi_set_ui(fib0, 1);
            mpfi_set_ui(fib1, 1);
            mpfi_set_ui(den, 1);


            for (k = 3; k <= N-4; k++)
              {
                mpfi_add(fibtmp, fib0, fib1);
                mpfi_mul(den, den, fibtmp);
                if (RIGHT(den)->_mpfr_exp > THRES_EXP)
                  {
                    if (flag)
                      mpfi_div(den, den, renorm);
                    else
                      {
                        flag = 1;
                        mpfi_init2(renorm, p);
                        mpfi_set_ui(renorm, 10);
                        mpfi_log(renorm, renorm);
                        mpfi_mul_ui(renorm, renorm,
                                    (unsigned long)(THRES_EXP *
                                                    0.30102999566398119521373889));
                        mpfi_exp(renorm, renorm);
                        mpfi_div(den, den, renorm);
                      }
                  }
                mpfi_swap(fib0, fib1);
                mpfi_swap(fib1, fibtmp);
              }

            mpfi_add(fibtmp, fib0, fib1); /* F_(N-3) */
            mpfi_swap(fib0, fib1);
            mpfi_swap(fib1, fibtmp);

            mpfi_add(fibtmp, fib0, fib1); /* F_(N-2) */
            mpfi_swap(fib0, fib1);
            mpfi_swap(fib1, fibtmp);

            mpfi_ui_div(num, 1, den);

            for (k = N - 1; k <= 2*N - 6; k++)
              {
                mpfi_add(fibtmp, fib0, fib1); /* F_k */
                mpfi_mul(num, num, fibtmp);
                /* If exponent grows too large, renormalize */
                if (RIGHT(num)->_mpfr_exp > THRES_EXP)
                  {
                    if (flag)
                      mpfi_div(num, num, renorm);
                    else
                      {
                        flag = 1;
                        mpfi_init2(renorm, p);
                        mpfi_set_ui(renorm, 10);
                        mpfi_log(renorm, renorm);
                        mpfi_mul_ui(renorm, renorm,
                                    (unsigned long)(THRES_EXP *
                                                    0.30102999566398119521373889));
                        mpfi_exp(renorm, renorm);
                        mpfi_div(num, num, renorm);
                      }
                  }
                mpfi_swap(fib0, fib1);
                mpfi_swap(fib1, fibtmp);
              }

            mpfi_mul(num, num, num);
            mpfi_div_ui(num, num, 2);

            mpfi_add(fibtmp, fib0, fib1); /* F_{2N-5} */
            mpfi_swap(fib0, fib1);
            mpfi_swap(fib1, fibtmp);

            mpfi_mul(num, num, fib1);

            mpfi_add(fibtmp, fib0, fib1); /* F_{2N-4} */
            mpfi_swap(fib0, fib1);
            mpfi_swap(fib1, fibtmp);

            mpfi_mul(num, num, fib1); mpfi_mul(num, num, fib1);

            mpfi_add(fibtmp, fib0, fib1); /* F_{2N-3} */
            mpfi_mul(num, num, fibtmp);
            mpfi_swap(fib0, fib1);
            mpfi_swap(fib1, fibtmp);

            mpfi_add(fibtmp, fib0, fib1); /* F_{2N-2} */
            mpfi_mul(num, num, fibtmp);
            if (N%2 == 1) mpfi_neg(num, num);

            lo = mpfr_get_str (NULL, &exp_lo, 10, 10, LEFT(num), GMP_RNDZ);
            hi = mpfr_get_str (NULL, &exp_hi, 10, 10, RIGHT(num), GMP_RNDZ);
            ok = exp_lo == exp_hi && strcmp (lo, hi) == 0;

            if (ok)
              break;
            else
              {
                p = p+p/10;
                fprintf(stderr, "Increasing precision to %lu.\n",
                        (unsigned long) p);
                mpfi_set_prec(num, p);
                mpfi_set_prec(den, p);
                mpfi_set_prec(fib0, p);
                mpfi_set_prec(fib1, p);
                mpfi_set_prec(fibtmp, p);
              }
          }
        while (1);

        mpfi_clear(num); mpfi_clear(den); mpfi_clear(fibtmp);
        mpfi_clear(fib0); mpfi_clear(fib1);
      }
      else
        {
          mpz_t expn, redarg;
          mpfr_t logphi, fib0, fib1, fibtmp;
          mpfr_t num, den, x, z, expnfr;
          mpz_init(redarg);
          mpz_init(expn);

          mpfr_init2(logphi, p);
          mpfr_init2(fib0, p);
          mpfr_init2(fib1, p);
          mpfr_init2(fibtmp, p);
          mpfr_init2(num, p);
          mpfr_init2(den, p);
          mpfr_init2(x, p);
          mpfr_init2(z, p);
          mpfr_init2(expnfr, p);

          /* Compute by hand terms up to 50 */
          mpfr_set_ui(fib0, 1, GMP_RNDN);
          mpfr_set_ui(fib1, 1, GMP_RNDN);
          mpfr_set_ui(den, 1, GMP_RNDN);

          for (k = 3; k <= 50; k++)
            {
              mpfr_add(fibtmp, fib0, fib1, GMP_RNDN);
              mpfr_mul(den, den, fibtmp, GMP_RNDN);
              mpfr_swap(fib0, fib1);
              mpfr_swap(fib1, fibtmp);
            }

          mpfr_mul(den, den, den, GMP_RNDN);
          mpfr_mul_ui(den, den, 2, GMP_RNDN);

          /*      expn = 2*N*N - 2*N + 2448;      */

          mpz_set_ui(expn, N);
          mpz_sub_ui(expn, expn, 1);
          mpz_mul_ui(expn, expn, N);
          mpz_mul_2exp(expn, expn, 1);
          mpz_add_ui(expn, expn, 2548);
          mpfr_set_z(expnfr, expn, GMP_RNDN);

          /* (1 + sqrt(5))/2 */
          mpfr_set_ui(x, 5, GMP_RNDN);
          mpfr_sqrt(x, x,  GMP_RNDN);
          mpfr_pow_ui(num, x, 105,GMP_RNDN); /* sqrt(5) ^ {105} */
          mpfr_add_ui(x, x, 1, GMP_RNDN);
          mpfr_div_2exp(x, x, 1, GMP_RNDN);

          /* log(phi) / log(10) */
          mpfr_log(logphi, x, GMP_RNDN);
          mpfr_set_ui(z, 10, GMP_RNDN);
          mpfr_log(z, z, GMP_RNDN);
          mpfr_div(logphi, z, logphi, GMP_RNDN);

          /* expn <- expn mod log(phi) / log(10) */
          mpfr_div(z, expnfr, logphi, GMP_RNDN);
          mpfr_floor(z, z);
          mpfr_mul(z, logphi, z, GMP_RNDN);

          mpfr_sub(expnfr, expnfr, z, GMP_RNDN);
          mpfr_pow(x, x, expnfr, GMP_RNDN);

          mpfr_div(x, x, num, GMP_RNDN);
          mpfr_div(x, x, den, GMP_RNDN);

          lo = mpfr_get_str (NULL, &exp_lo, 10, 10, x, GMP_RNDZ);

          mpfr_clear(num); mpfr_clear(den); mpfr_clear(fibtmp);
          mpfr_clear(fib0); mpfr_clear(fib1);
        }
    }
  else
    {
      mpz_t N;
      mpz_t expn, redarg;
      mpfr_t logphi, fib0, fib1, fibtmp;
      mpfr_t num, den, x, z, expnfr;
      unsigned long p;

      mpz_init_set_str(N, argv[1], 10);
      mpz_init(redarg);
      mpz_init(expn);
      p = 2*mpz_sizeinbase(N, 2) + 50;
      fprintf (stderr, "Setting initial precision to %lu\n",
               (unsigned long) p);

      mpfr_init2(logphi, p);
      mpfr_init2(fib0, p);
      mpfr_init2(fib1, p);
      mpfr_init2(fibtmp, p);
      mpfr_init2(num, p);
      mpfr_init2(den, p);
      mpfr_init2(x, p);
      mpfr_init2(z, p);
      mpfr_init2(expnfr, p);

      /* Compute by hand terms up to 50 */
      mpfr_set_ui(fib0, 1, GMP_RNDN);
      mpfr_set_ui(fib1, 1, GMP_RNDN);
      mpfr_set_ui(den, 1, GMP_RNDN);

      for (k = 3; k <= 50; k++)
        {
          mpfr_add(fibtmp, fib0, fib1, GMP_RNDN);
          mpfr_mul(den, den, fibtmp, GMP_RNDN);
          mpfr_swap(fib0, fib1);
          mpfr_swap(fib1, fibtmp);
        }

      mpfr_mul(den, den, den, GMP_RNDN);
      mpfr_mul_ui(den, den, 2, GMP_RNDN);

      /*      expn = 2*N*N - 2*N + 2448;      */

      mpz_set(expn, N);
      mpz_sub_ui(expn, expn, 1);
      mpz_mul(expn, expn, N);
      mpz_mul_2exp(expn, expn, 1);
      mpz_add_ui(expn, expn, 2548);
      mpfr_set_z(expnfr, expn, GMP_RNDN);

      /* (1 + sqrt(5))/2 */
      mpfr_set_ui(x, 5, GMP_RNDN);
      mpfr_sqrt(x, x,  GMP_RNDN);
      mpfr_pow_ui(num, x, 105,GMP_RNDN); /* sqrt(5) ^ {105} */
      mpfr_add_ui(x, x, 1, GMP_RNDN);
      mpfr_div_2exp(x, x, 1, GMP_RNDN);

      /* log(phi) / log(10) */
      mpfr_log(logphi, x, GMP_RNDN);
      mpfr_set_ui(z, 10, GMP_RNDN);
      mpfr_log(z, z, GMP_RNDN);
      mpfr_div(logphi, z, logphi, GMP_RNDN);

      /* expn <- expn mod log(phi) / log(10) */
      mpfr_div(z, expnfr, logphi, GMP_RNDN);
      mpfr_floor(z, z);
      mpfr_mul(z, logphi, z, GMP_RNDN);

      mpfr_sub(expnfr, expnfr, z, GMP_RNDN);
      mpfr_pow(x, x, expnfr, GMP_RNDN);

      mpfr_div(x, x, num, GMP_RNDN);
      mpfr_div(x, x, den, GMP_RNDN);

      lo = mpfr_get_str (NULL, &exp_lo, 10, 10, x, GMP_RNDZ);

      mpfr_clear(num); mpfr_clear(den); mpfr_clear(fibtmp);
      mpfr_clear(fib0); mpfr_clear(fib1);
    }

  printf ("%s\n", lo);
  fprintf (stderr, "Cputime: %dms\n", cputime () - st);
  return 0;
}
