#define USE_MPFI
#include "many.h"

#define THRES_EXP (1 << 25)

int
main (int argc, char *argv[])
{
  unsigned long N = atoi (argv[1]), k, ok, flag = 0;
  mp_prec_t p;
  char *lo, *hi;
  mp_exp_t exp_lo, exp_hi;
  mpfi_t fib0, fib1, fibtmp;
  mpfi_t num, den, renorm;
  int st;

  fprintf (stderr, "Using GMP %s and MPFR %s\n", gmp_version, mpfr_version);
  st = cputime ();

  /* initial estimate of the working precision */
  p = (argc > 2 ? atoi(argv[2]) :
       (N <= 10000000 ? 100 : 10000));
  fprintf (stderr, "Setting initial precision to %lu\n", p);


  mpfi_init2(fib0, p);
  mpfi_init2(fib1, p);
  mpfi_init2(fibtmp, p);
  mpfi_init2(den, p);
  mpfi_init2(num, p);

  do
    {
      mpfi_set_ui(fib0, 1);
      mpfi_set_ui(fib1, 1);
      mpfi_set_ui(den, 1);

      for (k = 3; k <= N-4; k++)
        {
          mpfi_add(fibtmp, fib0, fib1);
          mpfi_mul(den, den, fibtmp);
          if (RIGHT(den)->_mpfr_exp > THRES_EXP)
            {
              if (flag)
                mpfi_div(den, den, renorm);
              else
                {
                  flag = 1;
                  mpfi_init2(renorm, p);
                  mpfi_set_ui(renorm, 10);
                  mpfi_log(renorm, renorm);
                  mpfi_mul_ui(renorm, renorm,
                              (unsigned long)(THRES_EXP *
                                              0.30102999566398119521373889));
                  mpfi_exp(renorm, renorm);
                  mpfi_div(den, den, renorm);
                }
            }
          mpfi_swap(fib0, fib1);
          mpfi_swap(fib1, fibtmp);
        }

      mpfi_add(fibtmp, fib0, fib1); /* F_(N-3) */
      mpfi_swap(fib0, fib1);
      mpfi_swap(fib1, fibtmp);

      mpfi_add(fibtmp, fib0, fib1); /* F_(N-2) */
      mpfi_swap(fib0, fib1);
      mpfi_swap(fib1, fibtmp);

      mpfi_ui_div(num, 1, den);

      for (k = N - 1; k <= 2*N - 6; k++)
        {
          mpfi_add(fibtmp, fib0, fib1); /* F_k */
          mpfi_mul(num, num, fibtmp);
          /* If exponent grows too large, renormalize */
          if (RIGHT(num)->_mpfr_exp > THRES_EXP)
            {
              if (flag)
                mpfi_div(num, num, renorm);
              else
                {
                  flag = 1;
                  mpfi_init2(renorm, p);
                  mpfi_set_ui(renorm, 10);
                  mpfi_log(renorm, renorm);
                  mpfi_mul_ui(renorm, renorm,
                              (unsigned long)(THRES_EXP *
                                              0.30102999566398119521373889));
                  mpfi_exp(renorm, renorm);
                  mpfi_div(num, num, renorm);
                }
            }
          mpfi_swap(fib0, fib1);
          mpfi_swap(fib1, fibtmp);
        }

      mpfi_mul(num, num, num);
      mpfi_div_ui(num, num, 2);

      mpfi_add(fibtmp, fib0, fib1); /* F_{2N-5} */
      mpfi_swap(fib0, fib1);
      mpfi_swap(fib1, fibtmp);

      mpfi_mul(num, num, fib1);

      mpfi_add(fibtmp, fib0, fib1); /* F_{2N-4} */
      mpfi_swap(fib0, fib1);
      mpfi_swap(fib1, fibtmp);

      mpfi_mul(num, num, fib1); mpfi_mul(num, num, fib1);

      mpfi_add(fibtmp, fib0, fib1); /* F_{2N-3} */
      mpfi_mul(num, num, fibtmp);
      mpfi_swap(fib0, fib1);
      mpfi_swap(fib1, fibtmp);

      mpfi_add(fibtmp, fib0, fib1); /* F_{2N-2} */
      mpfi_mul(num, num, fibtmp);

      lo = mpfr_get_str (NULL, &exp_lo, 10, 10, LEFT(num), GMP_RNDZ);
      hi = mpfr_get_str (NULL, &exp_hi, 10, 10, RIGHT(num), GMP_RNDZ);
      ok = exp_lo == exp_hi && strcmp (lo, hi) == 0;

      if (ok)
        break;
      else
        {
          p = p+p/10;
          fprintf(stderr, "Increasing precision to %lu.\n",
                  (unsigned long) p);
          mpfi_set_prec(num, p);
          mpfi_set_prec(den, p);
          mpfi_set_prec(fib0, p);
          mpfi_set_prec(fib1, p);
          mpfi_set_prec(fibtmp, p);
        }
    }
  while (1);

  mpfi_clear(num); mpfi_clear(den); mpfi_clear(fibtmp);
  mpfi_clear(fib0); mpfi_clear(fib1);

  printf ("%s\n", lo);

  fprintf (stderr, "Cputime: %dms\n", cputime () - st);
  return 0;
}
