#define USE_MPFI
#include "many.h"
#include "gauss.h"

/* row and column of wanted element of the inverse
   (one less than in original problem, since indices start at 0, not 1)
*/
#define ROW(N) (N-1)-1
#define COL(N) (N)-1

/*
N     10 20 50 100 200 500 1000
pmin  40 39 42  43  42  46   46
*/

int
main (int argc, char *argv[])
{
  unsigned long N = atoi (argv[1]);
  mp_prec_t p;
  char *lo, *hi;
  mp_exp_t exp_lo, exp_hi;
  mpfi_t **A, *b;
  unsigned long i, j;
  int st, ok;
  mpz_t fib;


  fprintf (stderr, "Using GMP %s and MPFR %s\n", gmp_version, mpfr_version);
  st = cputime ();
  mpz_init (fib);

  /* initial estimate of the working precision */
  p = (mp_prec_t) (argc > 2) ? atoi (argv[2]) : 46;
  fprintf (stderr, "Setting initial precision to %lu\n", p);

  A = malloc (N * sizeof (mpfi_t*));
  b = malloc (N * sizeof (mpfi_t));
  for (i = 0; i < N; i++)
    {
      mpfi_init2 (b[i], p);
      A[i] = malloc (N * sizeof (mpfi_t));
      for (j = 0; j < N; j++)
        mpfi_init2 (A[i][j], p);
    }

  do
    {
      /* A[i][j] = 1/fib(i+j-1) + delta(i,g) */
      for (i = 0; i < N; i++)
        for (j = 0; j < N; j++)
          {
	    mpz_fib_ui (fib, i + j + 1);
            mpfi_set_z (A[i][j], fib);
            mpfi_ui_div (A[i][j], 1, A[i][j]);
	    if (i == j)
		mpfi_add_ui (A[i][j], A[i][j], 1);
          }
      for (i = 0; i < N; i++)
        mpfi_set_ui (b[i], (i == COL(N)) ? 1 : 0);

      ok = gauss (A, b, N) == 0;

      if (ok)
        {
          lo = mpfr_get_str (NULL, &exp_lo, 10, 10, LEFT(b[ROW(N)]), GMP_RNDZ);
          hi = mpfr_get_str (NULL, &exp_hi, 10, 10, RIGHT(b[ROW(N)]), GMP_RNDZ);
          ok = exp_lo == exp_hi && strcmp (lo, hi) == 0;
        }

      if (ok)
        break;
      else
        {
          p += p / 10;
          fprintf (stderr, "Increasing precision to %lu\n", p);
          for (i = 0; i < N; i++)
            {
              mpfi_set_prec (b[i], p);
              for (j = 0; j < N; j++)
                mpfi_set_prec (A[i][j], p);
            }

        }
    }
  while (1);

  for (i = 0; i < N; i++)
    {
      mpfi_clear (b[i]);
      for (j = 0; j < N; j++)
        mpfi_clear (A[i][j]);
      free (A[i]);
    }
  free (b);
  free (A);

  printf ("%s\n", lo);

  fprintf (stderr, "Cputime: %dms\n", cputime () - st);
  return 0;
}
