void
print_matrix (mpfi_t **A, unsigned long N)
{
  unsigned long i, j;

  for (i = 0; i < N; i++)
    for (j = 0; j < N; j++)
      {
        printf ("A[%lu,%lu]=", i, j);
        mpfi_out_str (stdout, 10, 0, A[i][j]);
        printf ("\n");
      }
}

void
print_vector (mpfi_t *b, unsigned long N)
{
  unsigned long i;

  for (i = 0; i < N; i++)
    {
      printf ("b[%lu]=", i);
      mpfi_out_str (stdout, 10, 0, b[i]);
      printf ("\n");
    }
}

/* solve A*x = b by Gaussian elimination
   - works in place
   - assumes all coefficients have same precision
   - return 0 iff no error occurred
*/
int
gauss (mpfi_t **A, mpfi_t *b, unsigned long n)
{
  unsigned long i, j, k, l;
  unsigned long *piv; /* piv[i] is the column chosen as pivot for row i */
  unsigned long *col; /* col[j] is nonzero iff column j was not already
                         chosen as pivot */
  mpfi_t r, t;
  int ret = 0;

  piv = malloc (n * sizeof (unsigned long));
  col = malloc (n * sizeof (unsigned long));
  for (j = 0; j < n; j++)
    col[j] = 1;

  mpfi_init2 (r, mpfi_get_prec (A[0][0]));
  mpfi_init2 (t, mpfi_get_prec (A[0][0]));
  
  for (i = 0; i < n; i++) /* process row i */
    {
      /* find pivot */
      for (j = 0; j < n && (col[j] == 0 || mpfi_has_zero (A[i][j]) > 0); j++);
      if (j == n)
        {
          fprintf (stderr, "Error, no valid pivot found\n");
          ret = -1;
          goto end;
        }
      col[j] = 0;
      piv[i] = j;
      
      /* eliminate pivot column in rows i+1...n-1 */
      for (k = i + 1; k < n; k++)
        {
          mpfi_div (r, A[k][j], A[i][j]);
          for (l = 1; l < n; l++)
            if (col[l])
              { /* A[k][l] <- A[k][l] - (A[k][j]/A[i][j]) * A[i][l] */
                mpfi_mul (t, r, A[i][l]);
                mpfi_sub (A[k][l], A[k][l], t);
              }
          mpfi_set_ui (A[k][j], 0); /* FIXME: useful? */
          mpfi_mul (t, r, b[i]);
          mpfi_sub (b[k], b[k], t);
        }
    }

  /* back substitution */
  for (i = n; i-- > 0;)
    {
      j = piv[i];
      /* next variable is j */
      /* value is b[j] */
      for (k = i + 1; k < n; k++)
        {
          l = piv[k];
          mpfi_mul (t, A[i][l], b[l]);
          mpfi_sub (b[j], b[j], t);
        }
      mpfi_div (b[j], b[j], A[i][j]);
    }

 end:
    free (col);
    free (piv);
    mpfi_clear (r);
    mpfi_clear (t);

    return ret;
}
